@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2017-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2017-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of Postorius.
 #
@@ -18,13 +18,18 @@
 #
 
 import re
+
 from django import forms
-from django.core.validators import validate_email
 from django.core.exceptions import ValidationError
-from django.utils.translation import ugettext_lazy as _
+from django.core.validators import validate_email
+from django.utils.translation import gettext_lazy as _
+
 from django_mailman3.lib.mailman import get_mailman_client
 
 from postorius.forms.fields import ListOfStringsField
+from postorius.forms.validators import validate_uuid_or_email
+from postorius.models import EmailTemplate, _email_template_help_text
+from postorius.utils import LANGUAGES
 
 
 ACTION_CHOICES = (
@@ -35,6 +40,20 @@ ACTION_CHOICES = (
     ("defer", _("Default processing")),
 )
 
+DIGEST_FREQUENCY_CHOICES = (
+    ("daily", _("Daily")),
+    ("weekly", _("Weekly")),
+    ("quarterly", _("Quarterly")),
+    ("monthly", _("Monthly")),
+    ("yearly", _("Yearly"))
+)
+
+ROSTER_VISIBILITY_CHOICES = (
+    ("moderators", _("Only mailinglist moderators")),
+    ("members", _("Only mailinglist members")),
+    ("public", _("Anyone")),
+    )
+
 
 EMPTY_STRING = ''
 
@@ -126,10 +145,13 @@ class ListSubscribe(forms.Form):
     """Form fields to join an existing list.
     """
 
-    email = forms.ChoiceField(
+    subscriber = forms.ChoiceField(
         label=_('Your email address'),
-        validators=[validate_email],
         widget=forms.Select(),
+        validators=[validate_uuid_or_email, ],
+        help_text=_(
+            'Subscribing via "Primary Address" will change subscription'
+            ' address when you change your primary address.'),
         error_messages={
             'required': _('Please enter an email address.'),
             'invalid': _('Please enter a valid email address.')})
@@ -137,10 +159,15 @@ class ListSubscribe(forms.Form):
     display_name = forms.CharField(
         label=_('Your name (optional)'), required=False)
 
-    def __init__(self, user_emails, *args, **kwargs):
+    def __init__(self, user_emails, user_id, primary_email, *args, **kwargs):
         super(ListSubscribe, self).__init__(*args, **kwargs)
-        self.fields['email'].choices = ((address, address)
-                                        for address in user_emails)
+        choices = list((address, address)
+                       for address in user_emails)
+        if primary_email and user_id:
+            choices.insert(
+                0,
+                (user_id, _('Primary Address ({})').format(primary_email)))
+        self.fields['subscriber'].choices = choices
 
 
 class ListAnonymousSubscribe(forms.Form):
@@ -177,9 +204,9 @@ SUBSCRIPTION_POLICY_CHOICES = (
 )
 
 
-class ListSubscriptionPolicyForm(ListSettingsForm):
+class MemberPolicyForm(ListSettingsForm):
     """
-    List subscription policy settings.
+    Policies related to members.
     """
     subscription_policy = forms.ChoiceField(
         label=_('Subscription Policy'),
@@ -190,8 +217,85 @@ class ListSubscriptionPolicyForm(ListSettingsForm):
                     'Moderate: Moderators will have to authorize '
                     'each subscription manually.\n'
                     'Confirm then Moderate: First subscribers have to confirm,'
-                    ' then a moderator '
-                    'needs to authorize.'))
+                    ' then a moderator needs to authorize.'))
+
+    unsubscription_policy = forms.ChoiceField(
+        label=_('Un-Subscription Policy'),
+        choices=SUBSCRIPTION_POLICY_CHOICES,
+        help_text=_('Open: Un-Subscriptions happen automatically\n'
+                    'Confirm: Subscribers need to confirm the un-subscription '
+                    'using an email sent to them\n'
+                    'Moderate: Moderators will have to authorize '
+                    'each un-subscription manually.\n'
+                    'Confirm then Moderate: First subscribers have to confirm,'
+                    ' then a moderator needs to authorize.'))
+
+
+class BounceProcessingForm(ListSettingsForm):
+    """List's bounce processing settings."""
+
+    process_bounces = forms.BooleanField(
+        widget=forms.RadioSelect(choices=((True, _('Yes')), (False, _('No')))),
+        required=False,
+        label=_('Process Bounces'),
+        help_text=_(
+            'Specifies whether or not this list should do automatic'
+            ' bounce processing.'))
+
+    bounce_score_threshold = forms.IntegerField(
+        min_value=0,
+        label=_('Bounce score threshold'),
+        required=False,
+        help_text=_(
+            'This is the bounce score above which a member\'s subscription '
+            ' will be automatically disabled. When the subscription is '
+            ' re-enabled, their bounce score will be reset to zero.'))
+
+    bounce_info_stale_after = forms.CharField(
+        label=_('Bounce info stale after'),
+        required=False,
+        help_text=_(
+            'The number of days after which a member\'s bounce information'
+            ' is considered stale. If no new bounces have been received in'
+            ' the interim, the bounce score is reset to zero.'
+            ' This value must be an integer. '))
+
+    bounce_notify_owner_on_disable = forms.BooleanField(
+        widget=forms.RadioSelect(choices=((True, _('Yes')), (False, _('No')))),
+        required=False,
+        label=_('Notify owner on disable'),
+        help_text=_(
+            'This option controls whether or not the list owner is notified'
+            ' when a member\'s subscription is automatically disabled due'
+            ' to their bounce threshold being reached. '))
+
+    bounce_notify_owner_on_removal = forms.BooleanField(
+        widget=forms.RadioSelect(choices=((True, _('Yes')), (False, _('No')))),
+        required=False,
+        label=_('Notify owner on removal'),
+        help_text=_(
+            'This option controls whether or not the list owner is '
+            'notified when a member is removed from the list after '
+            'their disabled notifications have been exhausted. '))
+
+    # forward_unrecognized_bounces_to =
+
+    bounce_you_are_disabled_warnings_interval = forms.CharField(
+        label=_('Bounce disabled warnings interval'),
+        required=False,
+        help_text=_(
+            'The number of days between each disabled notification.'))
+
+    bounce_you_are_disabled_warnings = forms.IntegerField(
+        min_value=0,
+        label=_('Bounce disable warnings'),
+        required=False,
+        help_text=_(
+            'The number of notices a disabled member will receive before'
+            ' their address is removed from the mailing list\'s roster. '
+            'Set this to 0 to immediately remove an address from the list'
+            ' once their bounce score exceeds the threshold. '
+            'This value must be an integer. '))
 
 
 class ArchiveSettingsForm(ListSettingsForm):
@@ -290,8 +394,7 @@ class MessageAcceptanceForm(ListSettingsForm):
             'the message.'))
     default_nonmember_action = forms.ChoiceField(
         widget=forms.RadioSelect(),
-        label=_('Default action to take when a non-member posts to the '
-                'list'),
+        label=_('Default action to take when a non-member posts to the list'),
         error_messages={
             'required': _("Please choose a default non-member action.")},
         required=True,
@@ -301,12 +404,22 @@ class MessageAcceptanceForm(ListSettingsForm):
             'is matched against the list of explicitly accepted, held, '
             'rejected (bounced), and discarded addresses. '
             'If no match is found, then this action is taken.'))
+    emergency = forms.BooleanField(
+        widget=forms.RadioSelect(choices=((True, _('Yes')), (False, _('No')))),
+        required=False,
+        label=_('Emergency Moderation'),
+        help_text=_(
+            'When this option is enabled, all list traffic is emergency'
+            ' moderated, i.e. held for moderation. Turn this option on when'
+            ' your list is experiencing a flamewar and you want a cooling off'
+            ' period. '),
+    )
     max_message_size = forms.IntegerField(
         min_value=0,
         label=_('Maximum message size'),
         required=False,
         help_text=_(
-            'The maximum allowed message size. '
+            'The maximum allowed message size in KB. '
             'This can be used to prevent emails with large attachments. '
             'A size of 0 disables the check.'))
     max_num_recipients = forms.IntegerField(
@@ -318,6 +431,65 @@ class MessageAcceptanceForm(ListSettingsForm):
             'This can be used to prevent mass mailings from being accepted. '
             'A value of 0 disables the check.'))
 
+    # TODO: Expose after this functionality actually works in Core.
+    # max_days_to_hold = forms.IntegerField(
+    #     min_value=0,
+    #     label=_('Discard held posts after'),
+    #     required=False,
+    #     help_text=_(
+    #         'No. of days after which held messages will be automatically'
+    #         ' discarded.'))
+
+    accept_these_nonmembers = ListOfStringsField(
+        label=_("Accept these non-members"),
+        required=False,
+        help_text=_(
+            'This is a list, one per line, of regexps matching '
+            'addresses that are allowed to post to this mailing list without'
+            ' subscribing to the list.'
+            ' Entries are regexps beginning with `^\' and are matched against'
+            ' the sender addresses in the message.'
+            ' While non-regexp addresses can be entered here, it is preferred'
+            ' to add the address as a nonmember and set the nonmember\'s '
+            'Moderation to Default Processing.'))
+
+    hold_these_nonmembers = ListOfStringsField(
+        label=_("Hold these non-members"),
+        required=False,
+        help_text=_(
+            'This is a list, one per line, of regexps matching '
+            'nonmember addresses, posts from which are held automatically.'
+            ' Entries are regexps beginning with `^\' and are matched against'
+            ' the sender addresses in the message.'
+            ' While non-regexp addresses can be entered here, it is preferred'
+            ' to add the address as a nonmember and set the nonmember\'s '
+            'Moderation to Hold.'))
+
+    reject_these_nonmembers = ListOfStringsField(
+        label=_("Reject these non-members"),
+        required=False,
+        help_text=_(
+            'This is a list, one per line, of regexps matching '
+            'nonmember addresses, posts from which are rejected with notice to'
+            ' the sender.'
+            ' Entries are regexps beginning with `^\' and are matched against'
+            ' the sender addresses in the message.'
+            ' While non-regexp addresses can be entered here, it is preferred'
+            ' to add the address as a nonmember and set the nonmember\'s '
+            'Moderation to Reject.'))
+
+    discard_these_nonmembers = ListOfStringsField(
+        label=_("Discard these non-members"),
+        required=False,
+        help_text=_(
+            'This is a list, one per line, of regexps matching '
+            'nonmember addresses, posts from which are discarded automatically'
+            '. Entries are regexps beginning with `^\' and are matched against'
+            ' the sender addresses in the message.'
+            ' While non-regexp addresses can be entered here, it is preferred'
+            ' to add the address as a nonmember and set the nonmember\'s '
+            'Moderation to Discard.'))
+
     def clean_acceptable_aliases(self):
         # python's urlencode will drop this attribute completely if an empty
         # list is passed with doseq=True. To make it work for us, we instead
@@ -346,6 +518,28 @@ class DigestSettingsForm(ListSettingsForm):
     """
     List digest settings.
     """
+    digests_enabled = forms.ChoiceField(
+        choices=((True, _('Yes')), (False, _('No'))),
+        widget=forms.RadioSelect,
+        required=False,
+        label=_('Enable Digests'),
+        help_text=_('Should Mailman enable digests for this MailingList?'),
+        )
+    digest_send_periodic = forms.ChoiceField(
+        choices=((True, _('Yes')), (False, _('No'))),
+        widget=forms.RadioSelect,
+        required=False,
+        label=_('Send Digest Periodically'),
+        help_text=_('Should Mailman send out digests periodically?'),
+        )
+    digest_volume_frequency = forms.ChoiceField(
+        choices=DIGEST_FREQUENCY_CHOICES,
+        widget=forms.RadioSelect,
+        required=False,
+        label=_('Digest Volume Frequency'),
+        help_text=_('At what frequency should Mailman increment the digest '
+                    'volume number and reset the issue number?'),
+        )
     digest_size_threshold = forms.DecimalField(
         label=_('Digest size threshold'),
         help_text=_('How big in Kb should a digest be before '
@@ -373,7 +567,7 @@ class DMARCMitigationsForm(ListSettingsForm):
             'The action to apply to messages From: a domain publishing a '
             'DMARC policy of reject or quarantine or to all messages if '
             'DMARC Mitigate unconditionally is True.'))
-    dmarc_mitigate_unconditionally = forms.TypedChoiceField(
+    dmarc_mitigate_unconditionally = forms.ChoiceField(
         choices=((True, _('Yes')), (False, _('No'))),
         widget=forms.RadioSelect,
         required=False,
@@ -399,25 +593,97 @@ class DMARCMitigationsForm(ListSettingsForm):
             'mitigation action of wrap message applies.'))
 
 
+PERSONALIZATION_CHOICES = (
+    ('none', _('None')),
+    ('individual', _('Individual')),
+    ('full', _('Full'))
+)
+
+PERSONALIZATION_CHOICES_HELP = _(
+    """
+    None: No personalization.
+
+    Individual: Everyone gets a unique copy of the message, and there are a \
+          few more substitution variables, but no headers are modified.
+
+    Full: All of the 'individual' personalization plus recipient header \
+          modification. """)
+
+FILTER_ACTION_CHOICES = (
+    ('discard', _('Discard')),
+    ('reject', _('Reject')),
+    ('forward', _('Forward')),
+    ('preserve', _('Preserve')),
+)
+
+FILTER_ACTION_HELP = _("""Action to take on messages which have no content
+after filtering.
+ Discard = silently discard the message.
+ Reject = discard the message and notify the sender.
+ Forward = forward the message to the list owner(s).
+ Preserve = save the message in qfiles/bad.
+""")
+
+
 class AlterMessagesForm(ListSettingsForm):
     """
     Alter messages list settings.
     """
-    filter_content = forms.TypedChoiceField(
+    personalize = forms.ChoiceField(
+        choices=PERSONALIZATION_CHOICES,
+        widget=forms.RadioSelect,
+        required=False,
+        label=_('Personalize'),
+        help_text=PERSONALIZATION_CHOICES_HELP)
+    filter_content = forms.ChoiceField(
         choices=((True, _('Yes')), (False, _('No'))),
         widget=forms.RadioSelect,
         required=False,
         label=_('Filter content'),
         help_text=_('Should Mailman filter the content of list traffic '
                     'according to the settings below?'))
-    collapse_alternatives = forms.TypedChoiceField(
+    filter_types = ListOfStringsField(
+        label=_('Filter types'),
+        required=False,
+        help_text=_(
+            'MimeTypes to filter from the incoming posts. A list of common '
+            'types can be found '
+            '<a href="https://developer.mozilla.org/en-US/docs/Web/HTTP/Basics_of_HTTP/MIME_types/Common_types">here </a>'  # noqa# E501
+            ))
+    filter_extensions = ListOfStringsField(
+        label=_('Filter extensions'),
+        required=False,
+        help_text=_(
+            'Extensions to filter from the incoming posts.'
+            ))
+    pass_types = ListOfStringsField(
+        label=_('Pass types'),
+        required=False,
+        help_text=_(
+            'MimeTypes to allow in the incoming posts. A list of common '
+            'types can be found '
+            '<a href="https://developer.mozilla.org/en-US/docs/Web/HTTP/Basics_of_HTTP/MIME_types/Common_types">here </a>'  # noqa# E501
+            ))
+    pass_extensions = ListOfStringsField(
+        label=_('Pass extensions'),
+        required=False,
+        help_text=_(
+            'Extensions to allow in the incoming posts.'
+            ))
+    collapse_alternatives = forms.ChoiceField(
         choices=((True, _('Yes')), (False, _('No'))),
         widget=forms.RadioSelect,
         required=False,
         label=_('Collapse alternatives'),
         help_text=_('Should Mailman collapse multipart/alternative to '
                     'its first part content?'))
-    convert_html_to_plaintext = forms.TypedChoiceField(
+    filter_action = forms.ChoiceField(
+        choices=FILTER_ACTION_CHOICES,
+        widget=forms.RadioSelect,
+        required=False,
+        label=_('Filter Action'),
+        help_text=FILTER_ACTION_HELP)
+    convert_html_to_plaintext = forms.ChoiceField(
         choices=((True, _('Yes')), (False, _('No'))),
         widget=forms.RadioSelect,
         required=False,
@@ -425,7 +691,7 @@ class AlterMessagesForm(ListSettingsForm):
         help_text=_('Should Mailman convert text/html parts to plain text? '
                     'This conversion happens after MIME attachments '
                     'have been stripped.'))
-    anonymous_list = forms.TypedChoiceField(
+    anonymous_list = forms.ChoiceField(
         choices=((True, _('Yes')), (False, _('No'))),
         widget=forms.RadioSelect,
         required=False,
@@ -433,7 +699,7 @@ class AlterMessagesForm(ListSettingsForm):
         help_text=_('Hide the sender of a message, '
                     'replacing it with the list address '
                     '(Removes From, Sender and Reply-To fields)'))
-    include_rfc2369_headers = forms.TypedChoiceField(
+    include_rfc2369_headers = forms.ChoiceField(
         choices=((True, _('Yes')), (False, _('No'))),
         widget=forms.RadioSelect,
         required=False,
@@ -451,7 +717,7 @@ class AlterMessagesForm(ListSettingsForm):
             'As a last resort you can disable these headers, but this is not '
             'recommended (and in fact, your ability to disable these headers '
             'may eventually go away).'))
-    allow_list_posts = forms.TypedChoiceField(
+    allow_list_posts = forms.ChoiceField(
         choices=((True, _('Yes')), (False, _('No'))),
         widget=forms.RadioSelect,
         required=False,
@@ -467,7 +733,7 @@ class AlterMessagesForm(ListSettingsForm):
             'This option allows admins to set an explicit Reply-to address. '
             'It is only used if the reply-to is set to use an explicitly set '
             'header'))
-    first_strip_reply_to = forms.TypedChoiceField(
+    first_strip_reply_to = forms.ChoiceField(
         choices=((True, _('Yes')), (False, _('No'))),
         widget=forms.RadioSelect,
         required=False,
@@ -504,9 +770,13 @@ class AlterMessagesForm(ListSettingsForm):
             'override the Reply-To: header. One is that some posters depend '
             'on their own Reply-To: settings to convey their valid return '
             'address. Another is that modifying Reply-To: makes it much more '
-            'difficult to send private replies. See `Reply-To\' Munging '
-            'Considered Harmful for a general discussion of this issue. '
-            'See Reply-To Munging Considered Useful for a dissenting opinion. '
+            'difficult to send private replies. See <a href="'
+            'http://marc.merlins.org/netrants/reply-to-harmful.html">'
+            '`Reply-To\' Munging Considered Harmful</a> for a general '
+            'discussion of this issue. See <a href="'
+            'http://marc.merlins.org/netrants/reply-to-useful.html">'
+            '`Reply-To\' Munging Considered Useful</a> for a dissenting '
+            'opinion. '
             'Some mailing lists have restricted '
             'posting privileges, with a parallel list devoted to discussions. '
             'Examples are `patches\' or `checkin\' lists, where software '
@@ -574,7 +844,7 @@ class ListAutomaticResponsesForm(ListSettingsForm):
             'or -request/-owner address from the same poster. Set to zero '
             '(or negative) for no grace period (i.e. auto-respond to every '
             'message).'))
-    respond_to_post_requests = forms.TypedChoiceField(
+    respond_to_post_requests = forms.ChoiceField(
         choices=((True, _('Yes')), (False, _('No'))),
         widget=forms.RadioSelect,
         required=False,
@@ -583,7 +853,7 @@ class ListAutomaticResponsesForm(ListSettingsForm):
             'Should Mailman notify users about their messages held for '
             'approval. If you say \'No\', no notifications will be sent '
             'to users about the pending approval on their messages.'))
-    send_welcome_message = forms.TypedChoiceField(
+    send_welcome_message = forms.ChoiceField(
         choices=((True, _('Yes')), (False, _('No'))),
         widget=forms.RadioSelect,
         required=False,
@@ -591,23 +861,20 @@ class ListAutomaticResponsesForm(ListSettingsForm):
         help_text=_(
             'Send welcome message to newly subscribed members? '
             'Turn this off only if you plan on subscribing people manually '
-            'and don\'t want them to know that you did so. This option is '
-            'most useful for transparently migrating lists from some other '
-            'mailing list manager to Mailman.'))
-    welcome_message_uri = forms.CharField(
-        label=_('URI for the welcome message'),
+            'and don\'t want them to know that you did so. Setting this to No '
+            'is most useful for transparently migrating lists from some other '
+            'mailing list manager to Mailman.\n'
+            'The text of Welcome message can be set via the Templates tab.'))
+    send_goodbye_message = forms.ChoiceField(
+        choices=((True, _('Yes')), (False, _('No'))),
+        widget=forms.RadioSelect,
         required=False,
+        label=_('Send goodbye message'),
         help_text=_(
-            'If a welcome message is to be sent to subscribers, you can '
-            'specify a URI that gives the text of this message.'),
-    )
-    goodbye_message_uri = forms.CharField(
-        label=_('URI for the good bye message'),
-        required=False,
-        help_text=_(
-            'If a good bye message is to be sent to unsubscribers, you can '
-            'specify a URI that gives the text of this message.'),
-    )
+            'Send goodbye message to newly unsubscribed members? '
+            'Turn this off only if you plan on unsubscribing people manually '
+            'and don\'t want them to know that you did so.\n'
+            'The text of Goodbye message can be set via the Templates tab.'))
     admin_immed_notify = forms.BooleanField(
         widget=forms.RadioSelect(choices=((True, _('Yes')), (False, _('No')))),
         required=False,
@@ -628,11 +895,18 @@ class ListAutomaticResponsesForm(ListSettingsForm):
                     'subscribes and unsubscribes?'))
 
 
+NEWSGROUP_MODERATION_CHOICES = (
+    ('none', _('Not Moderated')),
+    ('open_moderated', _('Moderated but allows for open posting')),
+    ('moderated', _('Moderated')),
+)
+
+
 class ListIdentityForm(ListSettingsForm):
     """
     List identity settings.
     """
-    advertised = forms.TypedChoiceField(
+    advertised = forms.ChoiceField(
         choices=((True, _('Yes')), (False, _('No'))),
         widget=forms.RadioSelect,
         label=_('Show list on index page'),
@@ -662,6 +936,54 @@ class ListIdentityForm(ListSettingsForm):
         strip=False,
         required=False,
     )
+    preferred_language = forms.ChoiceField(
+        label=_('Preferred Language'),
+        required=False,
+        widget=forms.Select(),
+        choices=LANGUAGES,
+    )
+    member_roster_visibility = forms.ChoiceField(
+        label=_('Members List Visibility'),
+        required=False,
+        widget=forms.Select(),
+        choices=ROSTER_VISIBILITY_CHOICES,
+        help_text=_('Who is allowed to see members list for this MailingList?')
+    )
+    gateway_to_mail = forms.BooleanField(
+        widget=forms.RadioSelect(choices=((True, _('Yes')), (False, _('No')))),
+        required=False,
+        label=_('Gateway to mail'),
+        help_text=_('Flag indicating that posts to the linked newsgroup should'
+                    ' be gated to the list')
+    )
+    gateway_to_news = forms.BooleanField(
+        widget=forms.RadioSelect(choices=((True, _('Yes')), (False, _('No')))),
+        required=False,
+        label=_('Gateway to news'),
+        help_text=_('Flag indicating that posts to the list should be gated to'
+                    ' the linked newsgroup.')
+    )
+    linked_newsgroup = forms.CharField(
+        label=_('Linked Newsgroup'),
+        required=False,
+        help_text=_(
+            'The name of the linked newsgroup.')
+    )
+    newsgroup_moderation = forms.ChoiceField(
+        label=_('Newsgroup moderation'),
+        required=False,
+        widget=forms.Select(),
+        choices=NEWSGROUP_MODERATION_CHOICES,
+        help_text=_('The moderation policy for the linked newsgroup,'
+                    ' if there is one.')
+    )
+    nntp_prefix_subject_too = forms.BooleanField(
+        widget=forms.RadioSelect(choices=((True, _('Yes')), (False, _('No')))),
+        required=False,
+        label=_('NNTP Include subject prefix '),
+        help_text=_('Flag indicating whether the list\'s "Subject Prefix"'
+                    ' should be included in posts gated to usenet.')
+    )
 
     def clean_subject_prefix(self):
         """
@@ -682,10 +1004,73 @@ class ListMassSubscription(forms.Form):
             'John Doe &lt;jdoe@example.com&gt;\n'
             '"John Doe" &lt;jdoe@example.com&gt;\n'
             'jdoe@example.com (John Doe)\n'
-            'Use the last three to associate a display name with'
-            ' the address\n'),
+            'Use the last three to associate a display name with the address\n'
+            ),
     )
 
+    pre_confirmed = forms.BooleanField(
+        label=_('Pre confirm'),
+        initial=True,
+        required=False,
+        help_text=_(
+            'If checked, users will not have to confirm their subscription.'),
+        widget=forms.CheckboxInput()
+        )
+
+    pre_approved = forms.BooleanField(
+        label=_('Pre approved'),
+        initial=True,
+        required=False,
+        help_text=_(
+            'If checked, moderators will not have to approve the subscription'
+            ' request.',),
+        widget=forms.CheckboxInput()
+        )
+
+    pre_verified = forms.BooleanField(
+        label=_('Pre Verified'),
+        initial=False,
+        required=False,
+        help_text=_(
+            'If checked, users will not have to verify that their '
+            'email address is valid.'),
+        widget=forms.CheckboxInput()
+        )
+
+    invitation = forms.BooleanField(
+        label=_('Invitation'),
+        initial=False,
+        required=False,
+        help_text=_(
+            'If checked, the other checkboxes are ignored and the users will '
+            'be sent an invitation to join the list and will be subscribed '
+            'upon acceptance thereof.'),
+        widget=forms.CheckboxInput()
+        )
+
+    send_welcome_message = forms.ChoiceField(
+        choices=((True, _('Yes')),
+                 (False, _('No')),
+                 ('default', _('List default'))),
+        widget=forms.RadioSelect,
+        initial='default',
+        required=False,
+        label=_('Send welcome message'),
+        help_text=_(
+            'If set to "Yes" or "No", List\'s default setting of '
+            'send_welcome_message will be ignored for these subscribers and a'
+            ' welcome message will be sent or not sent based on the choice.'),
+        )
+
+    def clean_send_welcome_message(self):
+        """Choose from True or False. Any other value is equivalent to None."""
+        data = self.cleaned_data['send_welcome_message']
+        if data in ('True', 'False'):
+            return data
+        # None implies this value is unset and isn't passed on to Core in API
+        # call.
+        return None
+
 
 class ListMassRemoval(forms.Form):
 
@@ -705,22 +1090,6 @@ class ListMassRemoval(forms.Form):
         layout = [["Mass Removal", "emails"]]
 
 
-class ListAddBanForm(forms.Form):
-    """Ban an email address for a list."""
-    # TODO maxking: This form should only accept valid emails or regular
-    # expressions. Anything else that doesn't look like a valid email address
-    # or regexp for email should not be a valid value for the field. However,
-    # checking for that might not be easy.
-    email = forms.CharField(
-        label=_('Add ban'),
-        help_text=_(
-            'You can ban a single email address or use a regular expression '
-            'to match similar email addresses.'),
-        error_messages={
-            'required': _('Please enter an email address.'),
-            'invalid': _('Please enter a valid email address.')})
-
-
 class ListHeaderMatchForm(forms.Form):
     """Edit a list's header match."""
 
@@ -787,17 +1156,45 @@ class MemberModeration(forms.Form):
             'Discard -- this simply discards the message, with no notice '
             'sent to the post\'s author. \n'
             'Accept -- accepts any postings without any further checks. \n'
-            'Defer -- default processing, run additional checks and accept '
+            'Default Processing -- run additional checks and accept '
             'the message. \n'))
 
 
 class ChangeSubscriptionForm(forms.Form):
-    email = forms.ChoiceField()
 
-    def __init__(self, user_emails, *args, **kwargs):
+    subscriber = forms.ChoiceField(
+        label=_('Select Email'),
+        required=False,
+        widget=forms.Select(),
+        validators=[validate_uuid_or_email, ],)
+
+    def __init__(self, user_emails, user_id, primary_email, *args, **kwargs):
         super(ChangeSubscriptionForm, self).__init__(*args, **kwargs)
-        self.fields['email'] = forms.ChoiceField(
-            label=_('Select Email'),
-            required=False,
-            widget=forms.Select(),
-            choices=((address, address) for address in user_emails))
+        choices = list((address, address)
+                       for address in user_emails)
+        if primary_email and user_id:
+            choices.insert(
+                0,
+                (user_id, _('Primary Address ({})').format(primary_email)))
+        self.fields['subscriber'].choices = choices
+
+
+class TemplateUpdateForm(forms.ModelForm):
+    data = forms.CharField(
+        label=_('Data'),
+        required=False,
+        strip=False,
+        widget=forms.Textarea(),
+        help_text=_email_template_help_text)
+
+    class Meta:
+        model = EmailTemplate
+        fields = ['data']
+
+
+class TokenConfirmForm(forms.Form):
+    """Form to confirm pending (un)subscription requests from User."""
+    token = forms.CharField(
+        required=True,
+        label="",
+        widget=forms.TextInput(attrs={'readonly': True, 'hidden': True}))
