@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2017-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2017-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of Postorius.
 #
@@ -19,9 +19,10 @@
 
 
 from django import forms
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 from postorius.forms.fields import NullBooleanRadioSelect
+from postorius.utils import LANGUAGES, with_empty_choice
 
 
 class UserPreferences(forms.Form):
@@ -66,7 +67,7 @@ class UserPreferences(forms.Form):
             're-enabled.'))
     delivery_mode = forms.ChoiceField(
         widget=forms.Select(),
-        choices=delivery_mode_choices,
+        choices=with_empty_choice(delivery_mode_choices),
         required=False,
         label=_('Delivery mode'),
         help_text=_(
@@ -108,8 +109,19 @@ class UserPreferences(forms.Form):
         help_text=_(
             'When you are listed explicitly in the To: or Cc: headers of a '
             'list message, you can opt to not receive another copy from the '
-            'mailing list. Select No to receive copies. '
-            'Select Yes to avoid receiving copies from the mailing list'))
+            'mailing list. Select Yes to receive copies. '
+            'Select No to avoid receiving copies from the mailing list'))
+
+    preferred_language = forms.ChoiceField(
+        widget=forms.Select(),
+        choices=with_empty_choice(LANGUAGES),
+        required=False,
+        label=_('Preferred language'),
+        help_text=_(
+            'Preferred language for your interactions with Mailman. When '
+            'this is set, it will override the MailingList\'s preferred '
+            'language. This affects which language is used for your '
+            'email notifications and such.'))
 
     class Meta:
 
@@ -119,7 +131,7 @@ class UserPreferences(forms.Form):
         """
         layout = [["User Preferences", "acknowledge_posts", "hide_address",
                    "receive_list_copy", "receive_own_postings",
-                   "delivery_mode", "delivery_status"]]
+                   "delivery_mode", "delivery_status", "preferred_language"]]
 
     def save(self):
         if not self.changed_data:
