@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2012-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2012-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of Postorius.
 #
@@ -16,12 +16,14 @@
 # Postorius.  If not, see <http://www.gnu.org/licenses/>.
 
 
-from allauth.account.models import EmailAddress
 from django.contrib.auth.models import User
 from django.urls import reverse
 
-from postorius.tests.utils import ViewTestCase
+from allauth.account.models import EmailAddress
+from django_mailman3.lib.mailman import get_mailman_user
+
 from postorius.forms import ListAnonymousSubscribe
+from postorius.tests.utils import ViewTestCase
 
 
 class ListSummaryPageTest(ViewTestCase):
@@ -38,7 +40,7 @@ class ListSummaryPageTest(ViewTestCase):
         self.user = User.objects.create_user(
             'testuser', 'test@example.com', 'testpass')
         EmailAddress.objects.create(
-            user=self.user, email=self.user.email, verified=True)
+            user=self.user, email=self.user.email, verified=True, primary=True)
 
     def test_list_summary_logged_out(self):
         # Response must contain list obj and anonymous subscribe form.
@@ -106,7 +108,7 @@ class ListSummaryPageTest(ViewTestCase):
         self.client.login(username='testuser', password='testpass')
         response = self.client.get(reverse('list_summary',
                                    args=('foo@example.com', )))
-        self.assertContains(response, 'Held messages</a>')
+        self.assertContains(response, 'Held messages')
         self.assertNotContains(response, 'Delete</a>')
 
     def test_list_summary_is_admin_secondary_owner(self):
@@ -136,7 +138,7 @@ class ListSummaryPageTest(ViewTestCase):
         response = self.client.get(reverse('list_summary',
                                    args=('foo@example.com', )))
         self.assertEqual(response.status_code, 200)
-        self.assertContains(response, 'Held messages</a>')
+        self.assertContains(response, 'Held messages')
         self.assertNotContains(response, 'Delete</a>')
 
     def test_metrics_not_displayed_to_anonymous(self):
@@ -170,3 +172,79 @@ class ListSummaryPageTest(ViewTestCase):
         response = self.client.get(reverse('list_summary',
                                            args=('foo@example.com',)))
         self.assertContains(response, 'List metrics')
+
+    def test_list_info(self):
+        # Test that list info is rendered as markdown.
+        settings = self.mm_client.get_list('foo@example.com').settings
+        info = 'Welcome To FooList today. This is something very interesting.'
+        settings['info'] = info
+        settings.save()
+        response = self.client.get(reverse('list_summary',
+                                           args=('foo@example.com',)))
+        self.assertContains(response, info)
+        settings['info'] = """\
+Welcome To Foolist
+==================
+
+```
+def function:
+    print('Hello World')
+```
+"""
+        settings.save()
+        response = self.client.get(reverse('list_summary',
+                                           args=('foo@example.com',)))
+        self.assertContains(response, '<pre><code>def function:')
+
+    def test_list_summary_already_subscribed(self):
+        self.foo_list.subscribe(
+            'test@example.com',
+            pre_verified=True, pre_confirmed=True, pre_approved=True)
+        self.client.login(username='testuser', password='testpass')
+        response = self.client.get(reverse('list_summary',
+                                           args=('foo@example.com', )))
+        # Assert two parts separately due to sometimes being newlines and stuff
+        # in the response HTML.
+        self.assertContains(response,
+                            'You are subscribed to this list with the '
+                            'following address:')
+        self.assertContains(response, ' <em>test@example.com</em>')
+
+    def test_list_summary_already_subscribed_user(self):
+        mm_user = get_mailman_user(self.user)
+        mm_user.addresses[0].verify()
+        mm_user.preferred_address = self.user.email
+        self.foo_list.subscribe(
+            str(mm_user.user_id),
+            pre_verified=True, pre_confirmed=True, pre_approved=True)
+        self.client.login(username='testuser', password='testpass')
+        response = self.client.get(reverse('list_summary',
+                                           args=('foo@example.com', )))
+        # Assert two parts separately due to sometimes being newlines and stuff
+        # in the response HTML.
+        self.assertContains(response,
+                            'You are subscribed to this list with the '
+                            'following address:')
+        self.assertContains(response,
+                            ' <em>Primary Address (test@example.com)</em>')
+
+    def test_list_summary_sets_preferred_address(self):
+        # Test that preferred address is set.
+        mm_user = get_mailman_user(self.user)
+        self.assertIsNone(mm_user.preferred_address)
+        self.client.login(username='testuser', password='testpass')
+        self.client.get(reverse('list_summary',
+                                args=('foo@example.com', )))
+        self.assertEqual(mm_user.preferred_address.email, self.user.email)
+
+    def test_list_summary_not_verified_does_not_set_preferred(self):
+        # Test that the preferred address is *not* set if the primary_email is
+        # not verified.
+        primary_email = EmailAddress.objects.create(
+            user=self.user, email='another@example.com', verified=False)
+        primary_email.set_as_primary()
+        mm_user = get_mailman_user(self.user)
+        self.assertIsNone(mm_user.preferred_address)
+        self.client.get(reverse('list_summary',
+                                args=('foo@example.com', )))
+        self.assertIsNone(mm_user.preferred_address)
