@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2018 by the Free Software Foundation, Inc.
+# Copyright (C) 2018-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of Postorius.
 #
@@ -19,14 +19,16 @@
 
 import urllib
 
-from allauth.account.models import EmailAddress
+from django.conf import settings
 from django.contrib.auth.models import User
 from django.contrib.sites.models import Site
 from django.urls import reverse
+
+from allauth.account.models import EmailAddress
 from django_mailman3.models import MailDomain
 
+from postorius.models import TEMPLATES_LIST, EmailTemplate
 from postorius.tests.utils import ViewTestCase
-from postorius.models import EmailTemplate, TEMPLATES_LIST
 
 
 class DomainTemplateViewTest(ViewTestCase):
@@ -50,12 +52,7 @@ class DomainTemplateViewTest(ViewTestCase):
         self.domain.add_owner('do@example.com')
 
     def tearDown(self):
-        try:
-            for template in self.domain.templates:
-                template.delete()
-        except urllib.error.HTTPError:
-            pass
-
+        EmailTemplate.objects.all().delete()
         # We do this because instead of returning an empty list, we get
         # a 404 error. This must be fixed in Client.
 
@@ -126,6 +123,24 @@ class DomainTemplateViewTest(ViewTestCase):
             name='list:admin:action:post',
             identifier='example.com').exists())
 
+    def test_template_create_view_whitespace(self):
+        self.client.login(
+            username='domain_owner', password='testpass')
+        # First, let's make sure we can GET the view.
+        url = reverse('domain_template_new', args=('example.com',))
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 200)
+        self.assertTrue('New Template' in str(response.content))
+        # Now, let's try to create a new Template with leading whitespace.
+        data = {'name': 'list:admin:action:post',
+                'data': '\n\n\t\tThis is test data.'}
+        response = self.client.post(url, data=data)
+        self.assertEqual(response.status_code, 302)
+        # Now, let's check to make sure that the data wasn't stripped.
+        templates = EmailTemplate.objects.get(
+            name='list:admin:action:post', identifier='example.com')
+        self.assertEqual(templates.data, '\n\n\t\tThis is test data.')
+
     def test_template_create_all(self):
         # Test that we can create all the template options.
         self.client.login(
@@ -140,7 +155,7 @@ class DomainTemplateViewTest(ViewTestCase):
             data = {'name': template_name,
                     'data': 'This is test data.'}
             response = self.client.post(url, data=data)
-            self.assertEqual(response.status_code, 302)
+            self.assertEqual(response.status_code, 302, template_name)
 
     def test_template_unique_property(self):
         # Test that we can create templates with uniqueness.
@@ -329,6 +344,25 @@ class MailingListTemplateViewTest(ViewTestCase):
             name='list:admin:action:post',
             identifier='fun.example.com').exists())
 
+    def test_template_create_view_whitespace(self):
+        self.client.login(
+            username='list_owner', password='testpass')
+        # First, let's make sure we can GET the view.
+        url = reverse('list_template_new', args=('fun.example.com',))
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 200)
+        self.assertTrue('Choose the template you want to customize.'
+                        in str(response.content))
+        # Now, let's try to create a new Template with leading whitespace.
+        data = {'name': 'list:admin:action:post',
+                'data': '\n\n\t\tThis is test data.'}
+        response = self.client.post(url, data=data)
+        self.assertEqual(response.status_code, 302)
+        # Now, let's check to make sure that the data wasn't stripped.
+        templates = EmailTemplate.objects.get(
+            name='list:admin:action:post', identifier='fun.example.com')
+        self.assertEqual(templates.data, '\n\n\t\tThis is test data.')
+
     def test_template_create_all(self):
         # Test that we can create all the template options.
         self.client.login(
@@ -339,7 +373,7 @@ class MailingListTemplateViewTest(ViewTestCase):
                     'data': 'This is test data.',
                     'language': 'en'}
             response = self.client.post(url, data=data)
-            self.assertEqual(response.status_code, 302)
+            self.assertEqual(response.status_code, 302, template_name)
 
     def test_template_unique_property(self):
         # Test that we can create templates with uniqueness.
@@ -394,10 +428,10 @@ class MailingListTemplateViewTest(ViewTestCase):
             name='list:admin:action:post',
             identifier='fun.example.com').exists())
 
-    def test_template_update_view(self):
+    def _test_template_update_view(self, template_name, updated_data):
         self.client.login(
             username='list_owner', password='testpass')
-        data = dict(name='list:admin:action:post',
+        data = dict(name=template_name,
                     context='list',
                     identifier='fun.example.com',
                     data='This is some new data.')
@@ -408,12 +442,18 @@ class MailingListTemplateViewTest(ViewTestCase):
         response = self.client.get(url)
         self.assertEqual(response.status_code, 200)
         # Now, let's try to update this template by POST'ing.
-        data = {'data': 'This is the update template data.'}
+        data = {'data': updated_data}
         response = self.client.post(url, data)
         self.assertEqual(response.status_code, 302)
         template = EmailTemplate.objects.get(pk=template.id)
         self.assertEqual(template.data, data['data'])
 
+    def test_template_update_view(self):
+        self._test_template_update_view('list:admin:action:post',
+                                        'This is the update template data.')
+        self._test_template_update_view('list:admin:action:subscribe',
+                                        '\n\n\n\tHello World')
+
 
 class TestTemplateAPIView(ViewTestCase):
 
@@ -425,18 +465,33 @@ class TestTemplateAPIView(ViewTestCase):
         self.mlist = self.domain.create_list('fun')
 
     def tearDown(self):
-        for each_level in (
-                self.domain.templates,
-                self.mlist.templates,
-                self.mm_client.templates
-        ):
-            try:
-                for template in each_level:
-                    template.delete()
-            except urllib.error.HTTPError:
-                pass
+        EmailTemplate.objects.all().delete()
         super().tearDown()
 
+    def test_get_empty_templates_via_API(self):
+        # Test that we can get a domain level template from API.
+        data = dict(name='list:admin:action:post',
+                    context='domain',
+                    identifier='example.com',
+                    data='')
+        # First, let's create a template.
+        EmailTemplate.objects.create(**data)
+        # Now, let's try to GET this template.
+        url = reverse(
+            'rest_template',
+            kwargs=dict(
+                name=data['name'],
+                context=data['context'],
+                identifier=data['identifier']
+                )
+            )
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.content, b'')
+        self.assertRegex(response["Content-Type"],
+                         'charset=' + settings.DEFAULT_CHARSET)
+        self.assertNotRegex(response["Content-Type"], 'charset=$')
+
     def test_get_one_domain_template_via_API(self):
         # Test that we can get a domain level template from API.
         data = dict(name='list:admin:action:post',
@@ -457,6 +512,9 @@ class TestTemplateAPIView(ViewTestCase):
         response = self.client.get(url)
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response.content, b'This is some new data.')
+        self.assertRegex(response["Content-Type"],
+                         'charset=' + settings.DEFAULT_CHARSET)
+        self.assertNotRegex(response["Content-Type"], 'charset=$')
 
     def test_get_one_list_template_via_API(self):
         # Test that we can get a list level template from API.
@@ -478,3 +536,6 @@ class TestTemplateAPIView(ViewTestCase):
         response = self.client.get(url)
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response.content, b'This is some other new data.')
+        self.assertRegex(response["Content-Type"],
+                         'charset=' + settings.DEFAULT_CHARSET)
+        self.assertNotRegex(response["Content-Type"], 'charset=$')
