@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2017-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2017-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of Postorius.
 #
@@ -17,10 +17,11 @@
 # Postorius.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-from django.test import TestCase
 from django.contrib.sites.models import Site
+from django.test import TestCase
 
-from postorius.forms import DomainEditForm, DomainForm
+from postorius.forms.domain_forms import (
+    DomainEditForm, DomainForm, DomainOwnerForm)
 
 
 class TestDomainEditForm(TestCase):
@@ -32,6 +33,15 @@ class TestDomainEditForm(TestCase):
         self.assertFalse('mail_host' in form.fields)
 
 
+class TestDomainOwnerAddForm(TestCase):
+    def test_form_sanity(self):
+        form = DomainOwnerForm(dict(email='some'))
+        self.assertFalse(form.is_valid())
+        self.assertIn('email', form.errors)
+        form = DomainOwnerForm(dict(email='some@example.com'))
+        self.assertTrue(form.is_valid())
+
+
 class TestDomainForm(TestCase):
     def test_form_contains_mail_host(self):
         form = DomainForm()
